"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FhApplication = void 0;
require("reflect-metadata");
var fh_forms_handler_1 = require("fh-forms-handler");
var fh_basic_controls_1 = require("fh-basic-controls");
var fh_charts_controls_1 = require("fh-charts-controls");
var fhdp_controls_1 = require("fhdp-controls");
var fhdp_extenders_1 = require("fhdp-extenders");
var FhApplication = /** @class */ (function () {
    /**
     * Private and used internally
     * @param config main configuration
     */
    function FhApplication(config) {
        var _this = this;
        this.__config = config;
        if (config.registerStandardModules) {
            this.registerModule(fh_forms_handler_1.FormsHandler);
            this.registerModule(fh_basic_controls_1.BasicControls);
        }
        if (config.registerChartsControls) {
            this.registerModule(fh_charts_controls_1.ChartsControls);
        }
        if (config.registerFHDPControls) {
            this.registerModule(fhdp_controls_1.FhDPControls);
        }
        if (config.additionalModules) {
            config.additionalModules.forEach(function (mod) {
                _this.registerModule(mod);
            });
        }
        if (config.liferay && config.liferay.enabled) {
            this.__liferay = config.liferay;
        }
    }
    /**
     * At first use must be called with `config` param, to properly init instance
     * Later uses can be without param, and it will return same instance as one inited at begin
     * @param config main configuration
     */
    FhApplication.getInstance = function (config) {
        if (!FhApplication.__instance && config) {
            FhApplication.__instance = new FhApplication(config);
        }
        else if (!FhApplication.__instance && !config) {
            console.error('Init instance first with config');
        }
        return FhApplication.__instance;
    };
    Object.defineProperty(FhApplication.prototype, "currentConnectionLostTranslation", {
        /**
         * Getter for connection lost translations
         */
        get: function () {
            var i18n = fh_forms_handler_1.FhContainer.get('I18n');
            return i18n.translateString('connectionLostRetry', undefined, i18n.selectedLanguage);
        },
        enumerable: false,
        configurable: true
    });
    FhApplication.prototype.isJSON = function (data) {
        try {
            return JSON.parse(data);
        }
        catch (e) {
            return false;
        }
    };
    /**
     * Getter for connector;
     * @returns Connector
     */
    FhApplication.prototype.getConnector = function () {
        return this.__connector;
    };
    /**
     * Registers new module Its alternative function to config node `additionalModules`
     * @param module an module that will be registered in fh app
     */
    FhApplication.prototype.registerModule = function (module) {
        new module().init();
    };
    /**
     * Registers callback for backend calls
     * @param name callback name (key)
     * @param callback actual callback function
     */
    FhApplication.prototype.registerCallback = function (name, callback) {
        var customActions = fh_forms_handler_1.FhContainer.get('CustomActions');
        if (customActions == null) {
            return new Error('CustomActions is not registered.');
        }
        customActions.callbacks[name] = callback;
    };
    /**
     * Function for initialise extenders
     */
    FhApplication.prototype.initExtenders = function () {
        var i18n = fh_forms_handler_1.FhContainer.get('I18n');
        new fhdp_extenders_1.FHDPExtender(__assign({ i18n: i18n }, this.__config.extensionsConfig));
    };
    /**
     * Adds translation string
     */
    FhApplication.prototype.addTranslation = function (lang, key, translation) {
        var _a;
        var i18n = fh_forms_handler_1.FhContainer.get('I18n');
        i18n.registerStrings(lang, (_a = {},
            _a[key] = translation,
            _a), true);
    };
    /**
     * get translation string
     */
    FhApplication.prototype.translate = function (key, args) {
        var i18n = fh_forms_handler_1.FhContainer.get('I18n');
        return i18n.translateString(key, args, i18n.selectedLanguage);
    };
    /**
     * Initialise translations for connection lost
     */
    FhApplication.prototype.initConnectionLostTranslations = function () {
        var _this = this;
        var i18n = fh_forms_handler_1.FhContainer.get('I18n');
        i18n.registerStrings('pl', {
            connectionLostRetry: 'Połączenie z serwerem zostało przerwane. Ponawiam próbę połączenia...'
        }, true);
        i18n.registerStrings('en', {
            connectionLostRetry: 'Connection with server is broken. Trying to reconnect...'
        }, true);
        if (this.__config.connectionLostTranslations) {
            Object.keys(this.__config.connectionLostTranslations).forEach(function (iso) {
                i18n.registerStrings(iso, {
                    connectionLostRetry: _this.__config.connectionLostTranslations[iso]
                }, true);
            });
        }
    };
    /**
     * Main init function
     * @param context app context for sockets Default 'socketForms'
     */
    FhApplication.prototype.init = function (context) {
        var _this = this;
        if (context === void 0) { context = 'socketForms'; }
        var util = fh_forms_handler_1.FhContainer.get('Util');
        var i18n = fh_forms_handler_1.FhContainer.get('I18n');
        if (this.__config.extensionsConfig) {
            this.initExtenders();
        }
        this.initConnectionLostTranslations();
        var contextPath;
        if (this.__liferay) {
            if (this.__liferay.fhBaseUrl != null) {
                contextPath = "" + this.__liferay.fhBaseUrl.replace('http:/', 'ws:/').replace('https:/', 'wss:/') + this.__liferay.fhContextPath + "/socketForms";
            }
            else {
                contextPath = this.__liferay.localPumaWsURL;
            }
        }
        else {
            contextPath = util.getPath(context);
        }
        this.__connector = fh_forms_handler_1.FhContainer.get("Connector")(contextPath, function () {
            fh_forms_handler_1.FhContainer.get('ApplicationLock')
                .createInfoDialog(_this.currentConnectionLostTranslation);
        }, function () {
            fh_forms_handler_1.FhContainer.get('ApplicationLock')
                .closeInfoDialog();
        });
        if (this.__liferay && window.Liferay) {
            this.__connector.incomingMessageCallback = function (data) {
                var isJSON = function (data) {
                    try {
                        return JSON.parse(data);
                    }
                    catch (e) {
                        return false;
                    }
                };
                var json = isJSON(data);
                console.log('incomingMessageCallback');
                if (window.Liferay.Session && (!json || json.eventType !== 'onTimer')) {
                    window.Liferay.Session.extend();
                }
            };
            this.__connector.outcomingMessageCallback = function (data) {
                var isJSON = function (data) {
                    try {
                        return JSON.parse(data);
                    }
                    catch (e) {
                        return false;
                    }
                };
                var json = isJSON(data);
                console.log('outcomingMessageCallback');
                if (window.Liferay.Session && (!json || json.eventType !== 'onTimer')) {
                    window.Liferay.Session.extend();
                }
                // after init discard any query parameters from url to prevent repeated actions (e.g. redirect from external page with action in parameter)
                if (json && json.command == 'Init') {
                    history.replaceState(history.state, document.title, location.origin + location.pathname);
                    if (parent) {
                        parent.history.replaceState(parent.history.state, parent.document.title, parent.location.origin + parent.location.pathname);
                    }
                }
            };
        }
        fh_forms_handler_1.FhContainer.rebind("Connector").toConstantValue(this.__connector);
        fh_forms_handler_1.FhContainer.get('SocketHandler').addConnector(this.__connector);
        try {
            var cookie = fhdp_extenders_1.getCookie();
            if (cookie && cookie['GUEST_LANGUAGE_ID']) {
                i18n.selectLanguage(cookie['GUEST_LANGUAGE_ID']);
                console.log("LANGUAGE SELECTED ON START(GUEST_LANGUAGE_ID): ", cookie['GUEST_LANGUAGE_ID']);
            }
            else if (cookie && cookie['USERLANG']) {
                i18n.selectLanguage(cookie['USERLANG']);
                console.log("LANGUAGE SELECTED ON START: ", cookie['USERLANG']);
            }
        }
        catch (e) {
            console.warn(e);
        }
        if (this.__liferay) {
            fh_forms_handler_1.FhContainer.get('FH').initExternal(this.__liferay.localPumaWsURL);
        }
        else {
            fh_forms_handler_1.FhContainer.get('FH').init();
        }
        this.registerCallback('callUcAction', function (actionName) {
            fh_forms_handler_1.FhContainer.get('ServiceManagerUtil').callAction('callUcAction', actionName);
        });
    };
    /**
     * Edits classlist of element provided by id
     * @param id Id of DOM element
     * @param remove class name that should be removed from element
     * @param add class name that should be added in element
     */
    FhApplication.prototype.createCallbackById = function (id, remove, add) {
        var element = document.getElementById(id);
        if (!!element) {
            if (!!remove) {
                element.classList.remove(remove);
            }
            if (!!add) {
                element.classList.add(add);
            }
        }
    };
    /**
     * Edits classlist of element wrapper provided by id
     * @param id Id of DOM element
     * @param remove class name that should be removed from element wrapper
     * @param add class name that should be added in element wrapper
     */
    FhApplication.prototype.createCallbackByWrapper = function (id, remove, add) {
        var element = document.getElementById(id);
        if (!!element) {
            var wrapper = element.parentNode;
            if (!!wrapper) {
                if (!!remove) {
                    wrapper.classList.remove(remove);
                }
                if (!!add) {
                    wrapper.classList.add(add);
                }
            }
        }
    };
    return FhApplication;
}());
exports.FhApplication = FhApplication;
//# sourceMappingURL=FhApplication.js.map